/***********\
 * graph.h *
\***********/

#include <thread.h>
#include <synch.h>
#include <errno.h>


/*======================*\
|* Optional definitions *|
\*======================*/


/*
#define SPECIFIC_INPUT      force the code to read files that specify
			    matched and unmatched edges
			    format of the imput file:
				p num_vert num_edges
				s seed
				c comentary
				a i j 0|1 (0 = free, 1 = matched)
                            If it is not defined the input file format is
                                p num_vert num_edges
                                s seed
                                c comentary
                                a i j     (1 <= i,j <= num_vert) */


/*=====================*\
|* General definitions *|
\*=====================*/


#define MAX_THREADS 20  /* Maximum number of threads, 
                           may be chanded freely        */
#define TRUE  1
#define FALSE 0

#define NORMAL  0       /* function exit: normal exit   */
#define ERROR  -1       /* function exit: abnormal exit */

#define ALLOC(X,Y) (Y*)calloc(X,sizeof(Y)) /* ALLOC (itens, type) */


#define MATCHED(X) graph_ptr->mate_ptr [graph_ptr->mate_ptr [X].mate].mate == X

#define VERT_FREE(X) graph_ptr->mate_ptr [graph_ptr->mate_ptr [X].mate].mate != X


/*=======================*\
|* Algorithm definitions *|
\*=======================*/


/* Manipulate the flag of a vertex. */

  /* questions */

#define IS_VISITED(X)     (X&1)
#define IS_EVEN(X)        (X&2)
#define IS_ODD(X)         (!(X&2))
#define IS_RIGHT(X)       (X&4)
#define IS_OUTER(X)       (X&8)
#define IS_BFS_ACTIVE(X)  (X&16)

  /* actions */

#define UNVISITED(X)      (X=0)
#define VISITED(X)        (X=X|1)
#define EVEN(X)           (X=X|2)
#define ODD(X)            (X=X&253)
#define RIGHT(X)          (X=X|4)
#define LEFT(X)           (X=X&251)
#define OUTER(X)          (X=X|8)
#define INNER(X)          (X=X&247)
#define BFS_ACTIVE(X)     (X=X|16)
#define BFS_INACTIVE(X)   (X=X&239)


/*==================*\
|* type definitions *|
\*==================*/


typedef unsigned int word;

typedef int boole;

  /* Edge node of the adjacency list */

typedef struct edge_node {

  word vert;                 /* adjacent vertex */

  struct edge_node *next;    /* implement the list of edges */

} edge;


  /* vertex array */

typedef struct vertex_node {

  edge *edge_ptr;           /* edges of the vertex */

} vertex;


/* used to make a list of free vertices */

typedef struct vert_free_node {

  word vert;

  struct vert_free_node *next;

} free_vert;


  /* mutex array, implement the mutual exclusion to each vertex */

typedef struct mutex_node {

  mutex_t mutex;

} mutex;


  /* mate array. The edge uv is matched if and only if
     mate(u) = v and mate (v) = u */

typedef struct mate_node {

  word mate;

} mate;


  /* level array. Indicate the level of the vertex 
     in the bread-first search */

typedef struct level_node {

  word level;

} level;


  /* predecessor array. Pointer to the father of a 
     vertex in the tree formed by the  bread-first
     search */

typedef struct pred_node {

  word pred;

} pred;


  /* blossom list. store the basic informatios of
     the blossoms */

typedef struct blossom_node {

  word left_peak, right_peak; /* left and right peaks */

  word base;                  /* base of the blossom  */

  struct blossom_node *next;  /* inplement  a  list.  */

} blossom;

  /* blossom array. Each vertex has a pointer to a blossom
     if it belongs to one. Obs: One vertex can  belong  to 
     only one blossom at a time. */

typedef struct bl_node {

  blossom *bl;

} bl;

  /* flag array for eache vertex. Each bit corresponds to
     an information.

      first  bit: 1 if the vertex is visited 2 unvisited
      second bit: 1 if the vertex is even 2 if odd
      third  bit: 1 if the vertex is right 2 if left
      forth  bit: 1 if the vertex is outer 2 if inner
      fifth  bit: 1 if the vertex is active 2 inactive */

typedef struct flag_node {

  char flag;

} flag;


  /* bread_first search array. Circular array that
     stores the vertex at the limit of the  search */

typedef struct bfs_node {

  word vert;

} bfs;


  /* struct that stores the basic information to do 
     a search localy at each thread */

typedef struct search_node {

  word index;                  /* Number of the processor */

  word size;                   /* size of the bread-first 
                                  search cirular list     */
  word begin;                  /* pointer to the beginnig
                                  of the informations at
                                  the circular list       */
  word end;                    /* pointer to the end of 
                                  the iformations at the
                                  circular list           */
  level *level_ptr;            /* level of each vertex    */

  pred *pred_ptr;              /* predecessors array      */

  flag *flag_ptr;              /* flag array              */

  blossom* blossom_ptr;        /* list of blossoms formed
                                  during the  actual
                                  search                  */
  bl *bl_ptr;                  /* array of pointer to the
                                  blossoms                */
  bfs *bfs_ptr;                /* circular list of BFS    */

} search;


  /* struct commom to all threads */

typedef struct graph_node {

  word num_vertices;         /* number of the vertices
                                of the graph           */
  vertex *vert_ptr;          /* array of vertices      */

  free_vert *free_vert_ptr;  /* list of free vertices  */

  mutex_t mut_free_vert;     /* mutex to the list of   
				free_vertices          */
  mutex *mutex_ptr;          /* array of mutex         */

  mate *mate_ptr;            /* matching array         */

} graph;


/*====================*\
|* library prototypes *|
\*====================*/

/* parse_command_line - get the parameters from the command line
   input - the number of paramenters and the respective strings
   obs - fill the global variables NUM_PROCESSORS, input_file_name
   output_file_name */

void parse_command_line (int, char * []);

/* add_edge - insert an edge into the graph structure
   input    - graph, vertices i and j, (1 <= i,j <= n) */

void add_edge (graph *, word, word);

/* maximal_matching - produces an initial matching non ordered manner
   input - the graph structure */

void maximal_matching (graph *);

/* print_graph - print the adjacency list
   input - the graph structure */

void print_graph (graph *);

/* statisctic_graph - inform the number of vertices matched
   input - the graph structure
   output - the number of matched edges */

word statistic_graph (graph *);

/* free_graph - free all memory used by the program
   input - the graph structure */

void free_graph (graph *);


/*===============*\
|* io prototypes *|
\*===============*/

/* input_graph - read the graph from the file and build the internal
		 representation
   input       - the name of the file containing the graph information */

graph *input_graph (void);

/* output_graph- write the matched edges in an output file
   input       - the name of the file containing the graph information */

void output_graph (graph *);

/*=====================*\
|* prlmatch prototypes *|
\*=====================*/


/* get_free_vert - get a free vertex from the free vertex list
   input - the graph structure
   output - a free vertex if it existe or zero otherwise */

word get_free_vert (graph *);

/* insert_bfs - insert the vertex into the circular list
   input - the search structure and the vertex */

void insert_bfs (search *, word);

/* get_bfs - get a vertex from the circular list
   input - the search structure
   output  - the next vertex of the bread-first search
   obs - return 0 (zero) if the list is empty */

word get_bfs (search *);

/* reverse_path - change the directions of the predecessors pointers
   between high and low.
   input - the searche structure the vertex high and the low */

void reverse_path (search *, word, word);

/* open_blossom - find the path through blossoms
   input - the search strucuture, the initial vertex belonging
   to a blossom B and a pointer to that blossom
   output - the base*(B) */

word open_blossom (search *, word, blossom *);

/* find_path - find the path between the vertices high and low
   input - the search structure, the vertices high and low and a pointer
   to a blossom indicating that a path through that blossom must
   be found */

void find_path (search *, word, word, blossom *);

/* augment_path - match the unmatched edges of the path and unmatch the others
   input - the graph, the search strucutres and the vertices high and low
   output - TRUE if the path was augmented, FALSE otherwise */

boole augment_path (graph *, search *, word, word);

/* base - return the base*(blossom (vert))
   input - the search structure and the vertex vert
   output - base*(blossom (vert)) */

word base (search *, word);

/* create_blossom - identify the vertices of the blossom and
   create a new one to the search strucuture
   input  - the search structure and the vertices s and t */

void create_blossom (search *, word, word);

/* search_algorithm - perform the search locally at the thread
   input - the graph structure, the searche structure and the root of
   the search */

boole search_algorithm (graph *, search *, word);

/* parallel_maximal_matching - each thread makes a greedy matching in the
   boundary informed
   input - the index of the processor,
	   the graph, and the boundary to work (begin...end) */

void parallel_maximal_matching (word, graph *, word, word);

/* parallel_matching - the function that will become a thread
   input - a structure thr_param */

void *parallel_matching (void *);

/* sequential_matching - garantee of maximum matching
   input - the graph */

void sequential_matching (graph *graph_ptr);

/* start_threads - start the threads and mark the time spent to
   solve the matching
   input - the graph structure */

void start_threads (graph *);


/*=================*\
|* sync prototypes *|
\*=================*/

/* sync_init - initialize the varibles to synchronize threads
   input - the number of threads */

void sync_init (int);

/* sync - synchronize threads */

void sync (void);

/* end_of_sync - release any state of the conditional variable and the mutex */

void end_of_sync (void);

/*====================*\
|* threads prototypes *|
\*====================*/

/* create_threads - create any threads as especified
   input - the number of threads to create the function to execute and an
           array of void arguments to each thread.
   output - SUCCESS or ERROR */

int create_threads (int, void *(*)(void *), void * []);

/* join - wait until all threads terminate */

void join (void);














