/*************************************************************************\
 * library.c - library of functions to manipulate adjaceny list of graph *
\*************************************************************************/

#include "graph.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <limits.h>


int NUM_PROCESSORS;

char input_file_name [256];

char output_file_name [256];


/* parse_command_line - get the parameters from the command line
   input - the number of paramenters and the respective strings 
   obs - fill the global variables NUM_PROCESSORS, input_file_name
         output_file_name */

void parse_command_line (int argc, char *argv []) {

  if (argc == 1 || argc > 4) {

    printf ("USAGE: prlmatch input CPUs [output]\n");
    exit (NORMAL);

  }

  strcpy (input_file_name, argv [1]);

  NUM_PROCESSORS = atoi (argv [2]);

  if (NUM_PROCESSORS < 1 || NUM_PROCESSORS > MAX_THREADS) {

    printf ("Wrong number of CPUs\n");
    printf ("USAGE: prlmatch input CPUs [output]\n");
    exit (NORMAL);
  
  }

  if (argc == 4)

    strcpy (output_file_name, argv [3]);

  else

    strcpy (output_file_name, "");

}


/* add_edge - insert an edge into the graph structure
   input    - graph, vertices i and j, (1 <= i,j <= n) */

void add_edge (graph *graph_ptr, word i, word j) {

  edge *edge_ptr;

  vertex *vert_ptr;


  /* create the edges */

  edge_ptr = ALLOC (1, edge);


  vert_ptr = &graph_ptr->vert_ptr [i];
  edge_ptr->vert = j;
  edge_ptr->next = vert_ptr->edge_ptr;
  vert_ptr->edge_ptr = edge_ptr;


  edge_ptr = ALLOC (1, edge);

  vert_ptr = &graph_ptr->vert_ptr [j];
  edge_ptr->vert = i;
  edge_ptr->next = vert_ptr->edge_ptr;
  vert_ptr->edge_ptr = edge_ptr;

}


/* maximal_matching - produces an initial matching non ordered manner
   input - the graph structure */

void maximal_matching (graph *graph_ptr) {

  word count;


  for (count = 1; count <= graph_ptr->num_vertices; count++) {

    if (VERT_FREE (count)) {

      edge *edge_ptr = graph_ptr->vert_ptr [count].edge_ptr;


      while (edge_ptr) {

        int adj = edge_ptr->vert;

        if (VERT_FREE (adj)) {

           graph_ptr->mate_ptr [count].mate = adj;
           graph_ptr->mate_ptr [adj].mate = count;

           break;
        }

        else 

          edge_ptr = edge_ptr->next;

      }

    }

  }

}


/* print_graph - print the adjacency list
   input - the graph structure */

void print_graph (graph *graph_ptr) {

  word count;

  for (count = 1; count <= graph_ptr->num_vertices; count++) {

    edge *edge_ptr = graph_ptr->vert_ptr [count].edge_ptr;


    printf ("(%d)", count);

    while (edge_ptr != NULL) {

      printf (" %d", edge_ptr->vert);

      edge_ptr = edge_ptr->next;

    }

    putchar ('\n');

  }

}


/* statisctic_graph - inform the number of vertices matched 
   input - the graph structure 
   output - the number of matched edges */

word statistic_graph (graph *graph_ptr) {

  word vert,

       match = 0;


  for (vert = 1; vert <= graph_ptr->num_vertices; vert++) 

    if (MATCHED (vert))

      match++;

  return match;

}


/* free_graph - free all memory used by the program 
   input - the graph structure */

void free_graph (graph *graph_ptr) {

  word count;


  for (count = 1; count <= graph_ptr->num_vertices; count++) {

    vertex *vert_ptr = &graph_ptr->vert_ptr [count];


    while (vert_ptr->edge_ptr != NULL) {

      edge *edge_ptr = vert_ptr->edge_ptr;


      vert_ptr->edge_ptr = vert_ptr->edge_ptr->next;

      free (edge_ptr);

    }

  }


  free (graph_ptr->vert_ptr);

  free (graph_ptr->mutex_ptr);

  free (graph_ptr->mate_ptr);

  free (graph_ptr);

}


