#include <stdio.h>
#include <stdlib.h>

#include <thread.h>
#include <synch.h>
#include <errno.h>

#include <time.h>
#include <sys/time.h>

#include "threads.h"

#define X 150
#define Y 150 


/* Parametro para os threads. 
   Fornece uma identificacao (Indice)
   para cada um. */

typedef struct {

  int index;

} thr_param;


int matriz_A [X] [Y],
    matriz_B [X] [Y],
    matriz_C [X] [Y];

/* Exclusao mutua par acessar
   os elementos da matriz */

mutex_t matriz_M [X] [Y];

/* Variaveis para medir tempos 
   dentro dos threads */

hrtime_t tab_start [20], tab_end [20],
         tab_vstart [20], tab_vend [20]; 

int num_cpu;

void *multiplica (void *input_ptr) {

  long int i,
	   j,
	   k;

  /* Parametro */

  thr_param *input = (thr_param *) input_ptr;


  /* Medir o tempo */

  tab_start [input->index] =  gethrtime ();

  tab_vstart [input->index] =  gethrvtime ();

  /* Multiplicar a matriz */

  for (i = 0; i < X; i++) 

    for (j = 0; j < Y; j++) 

      for (k = input->index; k < X; k += num_cpu) {
   
	mutex_lock (&matriz_M [i] [j]);
       
        matriz_C [i] [j] += matriz_A [i] [k] * matriz_B [k] [j]; 

	mutex_unlock (&matriz_M [i] [j]);

      }


  /* Terminar de medir o tempo */
 
  tab_end [input->index] =  gethrtime ();

  tab_vend [input->index] =  gethrvtime ();


  /* Sair o thread */

  thr_exit (0);

}


void main (int argc, char *argv []) {

  long c_start, c_end;
  time_t t_start, t_end;
  hrtime_t start, end; 


  /* Parametros para os threads */

  thr_param thr_input [15];
  void *input_ptr [15];
  

  int i,
      j,
      k;


  /* Inicializa os mutex da matriz */

  for (i = 0; i < X; i++)

    for (j = 0; j < Y; j++) {

      matriz_C [i] [j] = 0;

      mutex_init (&matriz_M [i] [j], USYNC_THREAD, NULL);

    }


  /* Atribui numeros randomicos */

  for (i = 0; i < X; i++)

    for (j = 0; j < Y; j++) {

      matriz_A [i] [j] = /*random () % 100*/i+j;

      matriz_B [i] [j] = /*random () % 100*/(X + Y) - (i + j);

    }


  /* Pega o numero de cpus */

  if (argc != 2) {

    printf ("mulmatriz NCPU\n");

    exit (0);

  }

  num_cpu = atoi (argv [1]);

  if (num_cpu < 1 || num_cpu > 8) {

    printf ("Numero errado de cpus (1..8)\n");

    exit (0);

  }


  /* Prepara paramentros */

  for (k = 0; k < num_cpu; k++) {

    thr_input [k].index = k;

    input_ptr [k] = &thr_input [k];

  }

 
  /* Cria threads */

  c_start = clock ();

  time (&t_start);

  start = gethrtime ();

  create_threads (num_cpu, multiplica, input_ptr);

  join ();

  c_end = clock ();

  time (&t_end);

  end = gethrtime ();


  printf ("Time gethrtime %f sec\n", ((float)(end - start)/1000000000));

  printf ("Time time %ld\n", (long) (t_end - t_start));

  printf ("Time clock %f\n", (float) c_end / 1000000.0); 

  for (i = 0; i < num_cpu; i++) {

    printf ("CPU %d hr %f hrv %f \n", i, ((float)(tab_end [i] - tab_start [i])/1e9), ((float)(tab_vend [i] - tab_vstart [i])/1e9)); 
    

  }

}

