/************************************\
 * sync.c - threads synchronization *
\************************************/

#include <thread.h>
#include <synch.h>
#include <errno.h>

static mutex_t mutex_block; /* mutex to cooperate with the conditional
			       variable */
static cond_t cond_block;   /* conditional variable to implement
			       synchronization */
static int count_block,     /* inform how many threads are blocked */

       num_threads;         /* Number of threads that will call sync */


/* sync_init - initialize the varibles to synchronize threads 
   input - the number of threads */

void sync_init (int num_thr) {

  count_block = num_threads = num_thr;

  mutex_init (&mutex_block, USYNC_THREAD, NULL); 

  cond_init (&cond_block, USYNC_THREAD, NULL);

}


/* sync - synchronize threads */

void sync (void) {

  mutex_lock (&mutex_block);

  --count_block;

  if (count_block)  

    cond_wait (&cond_block, &mutex_block);

  else {

    count_block = num_threads;

    cond_broadcast (&cond_block);

  }

  mutex_unlock (&mutex_block);

}


/* end_of_sync - release any state of the conditional variable and the mutex */

void end_of_sync (void) {

  cond_destroy (&cond_block);

  mutex_destroy (&mutex_block);

}
