/************************************************\
 * threads.c - library for threads manipulation *
\************************************************/

#include <thread.h>
#include <synch.h>
#include <errno.h>

#include "threads.h"

static int num_threads;                /* the number of threads created,
			                  used by function join */

static thread_t th_id [MAX_THREADS];   /* id's of the threads created */ 


/* create_threads - create any threads as especified 
   input - the number of threads to create the function to execute and an
	   array of void arguments to each thread.
   output - SUCCESS or ERROR */

int create_threads (int num_thr, void *(*func_ptr)(void *), void *arg []) {

  int i;


  num_threads = num_thr;

  if (arg) { /* the array of void arguments was especified */

    for (i = 0; i < num_threads; i++)  

      if (thr_create (NULL, 0, func_ptr, arg [i], THR_BOUND, &th_id [i]))
  
        return (ERROR);

  } 
  
  else { /* no argument was especified */

    for (i = 0; i < num_threads; i++)  

      if (thr_create (NULL, 0, func_ptr, NULL, THR_BOUND, &th_id [i]))

        return (ERROR);

  }

  return (SUCCESS);

}


/* join - wait until all threads terminate */

void join (void) {

  int i;


  for (i = 0; i < num_threads; i++) 

    thr_join (th_id [i], NULL, NULL);

}

